/*****************************************************************************

        DiffEvol.h
        Copyright (c) 2001-2003 Laurent de Soras

Algorithm by Kenneth Price and Rainer Storn
http://www.icsi.berkeley.edu/~storn/code.html

Based on an implementation by Olli Niemitalo and Magnus Jonsson.

This class implements Differential Evolution. It is a genetic algorithm aimed
to find the best set of values to minimize a "cost" function provided by the
client. See the D.E. web page for more information.

The cost function must be implemented through DiffEvolFtorInterface.

RandGen is a classic and very fast Linear Congruence Random Number Generator
which period is 2^32. If you plan to make huge DE, you will probably prefer
to change it against something stronger like the Mersenne Twister.

--- Legal stuff ---

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*Tab=3***********************************************************************/



#if ! defined (DiffEvol_HEADER_INCLUDED)
#define	DiffEvol_HEADER_INCLUDED

#if defined (_MSC_VER)
	#pragma once
	#pragma warning (4 : 4250) // "Inherits via dominance."
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"DiffEvolPopulation.h"
#include	"RandGen.h"

#include	<vector>



class DiffEvolFtorInterface;

class DiffEvol
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

						DiffEvol (int nbr_pop, const DiffEvolPopulation &min_arr, const DiffEvolPopulation &max_arr);
						DiffEvol (int nbr_pop, int nbr_var, const double min_arr [], const double max_arr []);
						DiffEvol (int nbr_pop, const DiffEvolPopulation &min_arr, const DiffEvolPopulation &max_arr, const DiffEvolPopulation &best_arr);
						DiffEvol (int nbr_pop, int nbr_var, const double min_arr [], const double max_arr [], const double best_arr []);
	virtual			~DiffEvol () {}

	double			evolve (DiffEvolFtorInterface &func, double gainbest, double gainr1, double gainr2, double gainr3, double cr);
	const DiffEvolPopulation &
						get_best_population () const;
	double			get_best_cost () const;



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

	class EvaluatedPopulation
	{
	public:
							EvaluatedPopulation ();

		DiffEvolPopulation
							_pop;
		double			_cost;
		bool				_valid_cost_flag;
	};

	typedef	std::vector <EvaluatedPopulation>	Generation;

	void				init (const double min_arr [], const double max_arr [], const double best_arr []);
	void				randomize_population (const double min_arr [], const double max_arr []);
	double			find_best (DiffEvolFtorInterface &func);

	int				_nbr_pop;			// Number of populations
	int				_nbr_var;			// Number of variables in population

	int				_best_pop;			// Negative if not yet determinated
	Generation		_cur_gen;
	Generation		_next_gen;

	RandGen			_rnd;



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

						DiffEvol ();
						DiffEvol (const DiffEvol &other);
	DiffEvol &		operator = (const DiffEvol &other);
	bool				operator == (const DiffEvol &other);
	bool				operator != (const DiffEvol &other);

};	// class DiffEvol



//#include	"DiffEvol.hpp"



#endif	// DiffEvol_HEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
