/*****************************************************************************

        RandGen.hpp
        Copyright (c) 2003 Laurent de Soras

Linear Congruence coefficients taken from the article "Tables of Linear
Congruential Generators of Different Sizes and Good Lattice Structure"
by P. L'Ecuyer in Mathematics of Computation vol 68 no. 225, p. 249,
available at http://www.iro.umontreal.ca/~lecuyer/papers.html

--- Legal stuff ---

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*Tab=3***********************************************************************/



#if defined (RandGen_CURRENT_CODEHEADER)
	#error Recursive inclusion of RandGen code header.
#endif
#define	RandGen_CURRENT_CODEHEADER

#if ! defined (RandGen_CODEHEADER_INCLUDED)
#define	RandGen_CODEHEADER_INCLUDED



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	<cassert>



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



RandGen::RandGen ()
:	_state (1)
{
}



void	RandGen::init (unsigned long seed)
{
	assert (seed >= 0);
	assert (seed <= 0xFFFFFFFFUL);

	_state = seed;
}



unsigned long	RandGen::gen_uint32 ()
{
	_state = (_state * 29943829UL + 17UL) & 0xFFFFFFFFUL;

	return (_state);
}



double	RandGen::gen_double ()
{
	const unsigned long	val_32 = gen_uint32 ();
	const double	mult = 1.0 / (65536.0 * 65536.0);
	const double	result = static_cast <double> (val_32) * mult;

	return (result);
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



#endif	// RandGen_CODEHEADER_INCLUDED

#undef RandGen_CURRENT_CODEHEADER



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
